@@ -124,7 +124,7 @@ int PMMG_pParMesh_to_ffmesh(const PMMG_pParMesh& mesh, Mesh3 *&T_TH3, bool distr
 class parmmg_Op : public E_F0mps {
  public:
   Expression eTh, xx, yy, zz;
-  static const int n_name_param = 33;
+  static const int n_name_param = 36;
   static basicAC_F0::name_and_type name_param[];
   Expression nargs[n_name_param];
 
@@ -144,6 +144,10 @@ class parmmg_Op : public E_F0mps {
     return nargs[i] ? GetAny< long >((*nargs[i])(stack)) : a;
   }
 
+  long arg(int i, Stack stack, bool a) const {
+    return nargs[i] ? GetAny< bool >((*nargs[i])(stack)) : a;
+  }
+
  public:
   parmmg_Op(const basicAC_F0 &args, Expression tth) : eTh(tth) {
     if (verbosity > 1) {
@@ -189,7 +193,10 @@ basicAC_F0::name_and_type parmmg_Op::name_param[] = {
 {"hgrad"             , &typeid(double)},/*!< [val], Control gradation */
 {"hgradreq"          , &typeid(double)},/*!< [val], Control gradation from required entities */
 {"ls"                , &typeid(double)},/*!< [val], Value of level-set */
-{"nodeCommunicators" , &typeid(KN<KN<long>>*)}
+{"requiredTriangle"  , &typeid(KN<long>*)},/*!< [val], References of surfaces with required triangles */
+{"localParameter"    , &typeid(KNM<double>*)},/*!< [val], Local parameters on given surfaces */
+{"nodeCommunicators" , &typeid(KN<KN<long>>*)},
+{"neighbors"         , &typeid(KN<long>*)}
 };
 
 class parmmg_ff : public OneOperator {
@@ -238,7 +245,7 @@ AnyType parmmg_Op::operator( )(Stack stack) const {
                     PMMG_ARG_dim,3,PMMG_ARG_MPIComm,comm,
                     PMMG_ARG_end);
 
-  KN< KN< long > >* communicators = nargs[32] ? GetAny< KN< KN< long > >* >((*nargs[32])(stack)) : 0;
+  KN< KN< long > >* communicators = nargs[34] ? GetAny< KN< KN< long > >* >((*nargs[34])(stack)) : 0;
   ffmesh_to_PMMG_pParMesh(Th, mesh, communicators != NULL);
 
   int root = mesh->info.root;
@@ -272,6 +279,46 @@ AnyType parmmg_Op::operator( )(Stack stack) const {
         }
       }
     }
+  KN< long > *prequiredTriangle = 0;
+  KNM< double > *plocalParameter = 0;
+  if (nargs[32]) {
+    prequiredTriangle = GetAny< KN< long > * >((*nargs[32])(stack));
+  }
+  if (nargs[33]) {
+    plocalParameter = GetAny< KNM< double > * >((*nargs[33])(stack));
+  }
+    if (prequiredTriangle && prequiredTriangle->N( ) > 0) {
+      const KN< long > &requiredTriangle = *prequiredTriangle;
+      std::sort(requiredTriangle + 0, requiredTriangle + requiredTriangle.N());
+      int nt;
+      if ( PMMG_Get_meshSize(mesh,NULL,NULL,NULL,&nt,NULL,NULL) !=1 ) {
+        exit(EXIT_FAILURE);
+      }
+      for (int k=1; k<=nt; k++) {
+        int ref, dummy;
+        if ( PMMG_Get_triangle(mesh,&dummy,&dummy,&dummy,
+                    &ref,NULL) != 1 ) {
+          exit(EXIT_FAILURE);
+        }
+        if (std::binary_search(requiredTriangle + 0, requiredTriangle + requiredTriangle.N(), ref)) {
+          if ( PMMG_Set_requiredTriangle(mesh,k) != 1 ) {
+            exit(EXIT_FAILURE);
+          }
+        }
+      }
+    }
+    if (plocalParameter && plocalParameter->M( ) > 0) {
+      const KNM< double > &localParameter = *plocalParameter;
+      ffassert(localParameter.N() == 4);
+      if ( PMMG_Set_iparameter(mesh,/*sol,*/PMMG_IPARAM_numberOfLocalParam,localParameter.M()) != 1 ) {
+        exit(EXIT_FAILURE);
+      }
+      for(int j = 0; j < localParameter.M(); ++j) {
+        if ( PMMG_Set_localParameter(mesh,/*sol,*/MMG5_Triangle,localParameter(0,j),localParameter(1,j),localParameter(2,j),localParameter(3,j)) != 1 ) {
+          exit(EXIT_FAILURE);
+        }
+      }
+    }
   }
 
   int i=2;
@@ -279,36 +326,36 @@ AnyType parmmg_Op::operator( )(Stack stack) const {
     PMMG_Set_iparameter(mesh,PMMG_IPARAM_verbose,       -1);
     PMMG_Set_iparameter(mesh,PMMG_IPARAM_mmgVerbose,    -1);
   }
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_verbose,       arg(i,stack,0L)); i++;   /*!< [-10..10], Tune level of verbosity */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_mmgVerbose,    arg(i,stack,0L)); i++;   /*!< [-10..10], Tune level of verbosity of Mmg */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_mem,           arg(i,stack,0L)); i++;   /*!< [n/-1], Set memory size to n Mbytes or keep the default value */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_debug,         arg(i,stack,0L)); i++;   /*!< [1/0], Turn on/off debug mode */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_mmgDebug,      arg(i,stack,0L)); i++;   /*!< [1/0], Turn on/off debug mode */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_angle,         arg(i,stack,0L)); i++;   /*!< [1/0], Turn on/off angle detection */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_iso,           arg(i,stack,0L)); i++;   /*!< [1/0], Level-set meshing */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_lag,           arg(i,stack,0L)); i++;   /*!< [-1/0/1/2], Lagrangian option */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_opnbdy,        arg(i,stack,0L)); i++;   /*!< [0/1], Enable preservation of open boundaries */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_optim,         arg(i,stack,0L)); i++;   /*!< [1/0], Optimize mesh keeping its initial edge sizes */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_optimLES,      arg(i,stack,0L)); i++;   /*!< [1/0], Strong mesh optimization for Les computations */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_noinsert,      arg(i,stack,0L)); i++;   /*!< [1/0], Avoid/allow point insertion */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_noswap,        arg(i,stack,0L)); i++;   /*!< [1/0], Avoid/allow edge or face flipping */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_nomove,        arg(i,stack,0L)); i++;   /*!< [1/0], Avoid/allow point relocation */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_nosurf,        arg(i,stack,0L)); i++;   /*!< [1/0], Avoid/allow surface modifications */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_anisosize,     arg(i,stack,0L)); i++;   /*!< [1/0], Turn on/off anisotropic metric creation when no metric is provided */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_octree,        arg(i,stack,0L)); i++;   /*!< [n], Specify the max number of points per octree cell (DELAUNAY) */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_meshSize,      arg(i,stack,0L)); i++;   /*!< [n], Target mesh size of Mmg (advanced use) */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_metisRatio,    arg(i,stack,0L)); i++;   /*!< [n], wanted ratio # mesh / # metis super nodes (advanced use) */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_ifcLayers,     arg(i,stack,0L)); i++;   /*!< [n], Number of layers of interface displacement */
-  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_groupsRatio,   arg(i,stack,0.)); i++;   /*!< [val], Allowed imbalance between current and desired groups size */
-  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_niter,         arg(i,stack,0L)); i++;   /*!< [n], Set the number of remeshing iterations */
-  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_angleDetection,arg(i,stack,0.)); i++;   /*!< [val], Value for angle detection */
-  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_hmin,          arg(i,stack,0.)); i++;   /*!< [val], Minimal mesh size */
-  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_hmax,          arg(i,stack,0.)); i++;   /*!< [val], Maximal mesh size */
-  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_hsiz,          arg(i,stack,0.)); i++;   /*!< [val], Constant mesh size */
-  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_hausd,         arg(i,stack,0.)); i++;   /*!< [val], Control global Hausdorff distance (on all the boundary surfaces of the mesh) */
-  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_hgrad,         arg(i,stack,0.)); i++;   /*!< [val], Control gradation */
-  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_hgradreq,      arg(i,stack,0.)); i++;   /*!< [val], Control gradation from required entities */
-  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_ls,            arg(i,stack,0.)); i++;   /*!< [val], Value of level-set */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_verbose,       arg(i,stack,0L));    i++;   /*!< [-10..10], Tune level of verbosity */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_mmgVerbose,    arg(i,stack,0L));    i++;   /*!< [-10..10], Tune level of verbosity of Mmg */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_mem,           arg(i,stack,0L));    i++;   /*!< [n/-1], Set memory size to n Mbytes or keep the default value */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_debug,         arg(i,stack,false)); i++;   /*!< [1/0], Turn on/off debug mode */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_mmgDebug,      arg(i,stack,false)); i++;   /*!< [1/0], Turn on/off debug mode */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_angle,         arg(i,stack,false)); i++;   /*!< [1/0], Turn on/off angle detection */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_iso,           arg(i,stack,false)); i++;   /*!< [1/0], Level-set meshing */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_lag,           arg(i,stack,0L));    i++;   /*!< [-1/0/1/2], Lagrangian option */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_opnbdy,        arg(i,stack,false)); i++;   /*!< [0/1], Enable preservation of open boundaries */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_optim,         arg(i,stack,false)); i++;   /*!< [1/0], Optimize mesh keeping its initial edge sizes */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_optimLES,      arg(i,stack,false)); i++;   /*!< [1/0], Strong mesh optimization for Les computations */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_noinsert,      arg(i,stack,false)); i++;   /*!< [1/0], Avoid/allow point insertion */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_noswap,        arg(i,stack,false)); i++;   /*!< [1/0], Avoid/allow edge or face flipping */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_nomove,        arg(i,stack,false)); i++;   /*!< [1/0], Avoid/allow point relocation */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_nosurf,        arg(i,stack,false)); i++;   /*!< [1/0], Avoid/allow surface modifications */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_anisosize,     arg(i,stack,false)); i++;   /*!< [1/0], Turn on/off anisotropic metric creation when no metric is provided */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_octree,        arg(i,stack,0L));    i++;   /*!< [n], Specify the max number of points per octree cell (DELAUNAY) */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_meshSize,      arg(i,stack,0L));    i++;   /*!< [n], Target mesh size of Mmg (advanced use) */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_metisRatio,    arg(i,stack,0L));    i++;   /*!< [n], wanted ratio # mesh / # metis super nodes (advanced use) */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_ifcLayers,     arg(i,stack,0L));    i++;   /*!< [n], Number of layers of interface displacement */
+  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_groupsRatio,   arg(i,stack,0.));    i++;   /*!< [val], Allowed imbalance between current and desired groups size */
+  if (nargs[i]) PMMG_Set_iparameter(mesh,PMMG_IPARAM_niter,         arg(i,stack,0L));    i++;   /*!< [n], Set the number of remeshing iterations */
+  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_angleDetection,arg(i,stack,0.));    i++;   /*!< [val], Value for angle detection */
+  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_hmin,          arg(i,stack,0.));    i++;   /*!< [val], Minimal mesh size */
+  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_hmax,          arg(i,stack,0.));    i++;   /*!< [val], Maximal mesh size */
+  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_hsiz,          arg(i,stack,0.));    i++;   /*!< [val], Constant mesh size */
+  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_hausd,         arg(i,stack,0.));    i++;   /*!< [val], Control global Hausdorff distance (on all the boundary surfaces of the mesh) */
+  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_hgrad,         arg(i,stack,0.));    i++;   /*!< [val], Control gradation */
+  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_hgradreq,      arg(i,stack,0.));    i++;   /*!< [val], Control gradation from required entities */
+  if (nargs[i]) PMMG_Set_dparameter(mesh,PMMG_DPARAM_ls,            arg(i,stack,0.));    i++;   /*!< [val], Value of level-set */
 
   if(communicators != NULL) {
     /* Set API mode */
@@ -344,7 +391,26 @@ AnyType parmmg_Op::operator( )(Stack stack) const {
 
   int ier = communicators == NULL ? PMMG_parmmglib_centralized(mesh) : PMMG_parmmglib_distributed(mesh);
   if( ier == PMMG_STRONGFAILURE ) exit(EXIT_FAILURE);
+  KN< long > *pneighbors = 0;
+  if (nargs[35]) {
+    pneighbors = GetAny< KN< long > * >((*nargs[35])(stack));
+    int icomm,i;
+    int n_node_comm_out;
+    int nitem_node_comm_out;
+    int color_node_out;
 
+    /* Get number of node interfaces */
+    ier = PMMG_Get_numberOfNodeCommunicators(mesh,&n_node_comm_out);
+    pneighbors->resize(n_node_comm_out);
+
+    /* Get outward proc rank and number of nodes on each interface */
+    for( icomm = 0; icomm < n_node_comm_out; icomm++ ) {
+      ier = PMMG_Get_ithNodeCommunicatorSize(mesh, icomm,
+                                             &color_node_out,
+                                             &nitem_node_comm_out);
+      pneighbors->operator[](icomm) = color_node_out;
+    }
+  }
   Mesh3 *Th_T = nullptr;
 
   PMMG_pParMesh_to_ffmesh(mesh, Th_T, communicators != NULL);
