@@ -33,7 +33,7 @@
 #include <fstream>
 #include <iostream>
 #include <cstring>
-#include "libmesh5.h"
+#include "libmeshb7.h"
 #include "ufunction.hpp"
 #include "error.hpp"
 #include "RNM.hpp"
@@ -502,7 +502,7 @@ namespace Fem2D
                         iv[jj] = this->operator()(K[jj]);
                     }
                     for (int eh=0;eh<6;eh++){
-                        if(verbosity>2) cout << "tetrahedra: " << k << " edge : " << eh << " lenght "<<  this->elements[k].lenEdge(eh) << endl;
+                        if(verbosity>2) cout << "tetrahedra: " << k << " edge : " << eh << " length "<<  this->elements[k].lenEdge(eh) << endl;
                     }
                     if(verbosity>2) cout << " A tetrahedra with a very small edge was created " << endl;
                     return 1;
@@ -516,7 +516,7 @@ namespace Fem2D
                 if( this->be(k).lenEdge(e) < Norme2(Psup-Pinf)/1e9 )
                 {
                     for (int eh=0;eh<3;e++){
-                        cout << "triangles: " << k << " edge : " << eh << " lenght "<<  this->be(k).lenEdge(e) << endl;
+                        cout << "triangles: " << k << " edge : " << eh << " length "<<  this->be(k).lenEdge(e) << endl;
                     }
                     cout << " A triangle with a very small edges was created " << endl;
                     return 1;
@@ -642,12 +642,12 @@ namespace Fem2D
             
             //&this = new Mesh3(newvertex,this->nt,this-nbe,vvv,tt,bb);
             
-            delete [] newNumero_Som;
+            //delete [] newNumero_Som;  debile !!!
         }
         else{
             cout << " no need to change the mesh " << endl;
         }
-        delete [] Numero_Som;
+        //delete [] Numero_Som;debile !!!
         
         
         BuildBound();
@@ -760,7 +760,10 @@ namespace Fem2D
     int Mesh3::load(const string & filename)
     {
         int bin;
-        int ver,inm,dim,err=0;
+        int ver;
+        int64_t inm;
+        int dim;
+        int err=0;
         int lf=filename.size()+20;
         KN<char>  fileb(lf),filef(lf);
         char *data = new char[filename.size()+1];
@@ -1047,7 +1050,7 @@ namespace Fem2D
            cerr << " WARNING!!! The mesh file just contains a set of vertices" << endl;
            
         if(nt==0 && nbe) {
-            cerr << " ERROR!!! The old SURFACE mesh3 is obselete, please use meshS type" << endl;
+            cerr << " ERROR!!! The old SURFACE mesh3 is obsolete, please use meshS type" << endl;
          ffassert(0);
         }
         
@@ -1100,7 +1103,7 @@ namespace Fem2D
             cerr << " WARNING!!! The mesh file just contains a set of vertices" << endl;
         
         if(nt==0 && nbe) {
-            cerr << " ERROR!!! The old SURFACE mesh3 is obselete, please use meshS type" << endl;
+            cerr << " ERROR!!! The old SURFACE mesh3 is obsolete, please use meshS type" << endl;
             ffassert(0);
         }
         
@@ -1147,7 +1150,8 @@ namespace Fem2D
     
     int Mesh3::Save(const string & filename) const
     {
-        int ver = GmfDouble, outm;
+        int ver = GmfDouble;
+        int64_t outm;
         if ( !(outm = GmfOpenMesh(filename.c_str(),GmfWrite,ver,3)) ) {
             cerr <<"  -- Mesh3::Save  UNABLE TO OPEN  :"<< filename << endl;
             return(1);
@@ -1205,7 +1209,8 @@ namespace Fem2D
     
     int Mesh3::SaveSurface(const string & filename) const
     {
-        int ver = GmfDouble, outm;
+        int ver = GmfDouble;
+        int64_t outm;
         if ( !(outm = GmfOpenMesh(filename.c_str(),GmfWrite,ver,3)) )
         {
             cerr <<"  -- Mesh3::Save  UNABLE TO OPEN  :"<< filename << endl;
@@ -1417,7 +1422,8 @@ namespace Fem2D
         vertices = vv;
         elements = tt;
         borderelements = bb;
-        
+        double cpu0 = ((double) clock())/CLOCKS_PER_SEC;
+     
         // check the input mesh file format
         if(verbosity && nv==0) {
             cerr << " WARNING! The mesh file doesn't contain any vertex" << endl;
@@ -1432,33 +1438,55 @@ namespace Fem2D
       
         mes=0.;
         mesb=0.;
-        
+        int err = 0;
         for (int i=0;i<nt;i++)
-            mes += this->elements[i].mesure();
+        {
+            double mese =this->elements[i].mesure();
+            if( mese <=0 ) {
+                
+                if( verbosity<2 && err < verbosity)
+                    cerr << " Bad orientation of tet  before cleanmesh "<< i << " mes = "<< mese << endl;
+                    err++;
+            }
+            mes += mese;
+        }
         
         for (int i=0;i<nbe;i++)
             mesb += this->be(i).mesure();
         
         if (cleanmesh){
+            err=0;
             if(verbosity>5)
             cout << "before clean mesh3, nv: " <<nv << " nt:" << nt << " nbe:" << nbe << endl;
             clean_mesh(precis_mesh, nv, nt, nbe, vertices, elements, borderelements, removeduplicate, rebuildboundary, orientation);
             if(verbosity>5)
                 cout << "after clean mesh3, nv: " <<nv << " nt:" << nt << " nbe:" << nbe << endl;
         }
-        
+        if(err)
+        {
+            cerr << " erreur read  mesh " << err << " negative  tet";
+            ErrorExec(" mesh with negative Tets",1);
+        }
         //  Add FH to be consitant we all constructor ...  July 09
         BuildBound();
+        double cpu1 = ((double) clock())/CLOCKS_PER_SEC;
+        double cpu2=cpu1,cpu3=cpu1,cpu4=cpu1,cpu5=cpu1;
         if(nt && nbe) {
             BuildAdj();
+            cpu2 = ((double) clock())/CLOCKS_PER_SEC;
             Buildbnormalv();
+            cpu3 = ((double) clock())/CLOCKS_PER_SEC;
             BuildjElementConteningVertex();
+            cpu4 = ((double) clock())/CLOCKS_PER_SEC;
             BuildGTree();
+            cpu5 = ((double) clock())/CLOCKS_PER_SEC;
         }
         //  end add
         if(verbosity>1)
             cout << "  -- End of read: mesure = " << mes << " border mesure " << mesb << endl;
-        
+        if(verbosity>3)
+            cout << "      timers Mesh3 :"<< cpu1-cpu0 << " "<< cpu2-cpu1
+                  << " " << cpu3-cpu2 <<     " " << cpu4-cpu3 <<  " " << cpu5-cpu4 <<endl;
         assert(mes>=0.);
     }
     
