@@ -30,11 +30,6 @@ extern long verbosity ;
 extern long searchMethod; //pichon
 extern bool lockOrientation;
 
-//  FOR M_PI
-#ifdef __STRICT_ANSI__
-#undef __STRICT_ANSI__
-#endif
-
 #include <cmath>
 #include  <cfloat>
 #include <cstdlib>
@@ -214,7 +209,7 @@ public:
 			k = j0+j1; // code of current edge
 			int jm = Min(j0,j1), NbAdj=0, He=-1;
 			int *pm=Head+k;
-			while (*pm>=0) // be carefull
+			while (*pm>=0) // be careful
 			{
 			    int m=*pm,jj=m%3,ii=m/3, jj0,jj1;
 			    VerticesNumberOfEdge(triangles[ii],jj,jj0,jj1);
@@ -789,7 +784,7 @@ int  WalkInTriangle(const Mesh & Th,int it, double *lambda,
 
 	if (k==2) // 2
 	{
-	    // let j be the vertex beetween the 2 edges
+	    // let j be the vertex between the 2 edges
 	    int j = 3-neg[0]-neg[1];
 	    R S = Area2(P,PF,Q[j]);
 
@@ -1016,10 +1011,12 @@ int Walk(const Mesh & Th,int& it, R *l,
                 gp  << "\n\n";
             }
         }
+        const double pi=3.141592653589793238463;//M_PI;
+
         for(int i=0; i<P.N(); ++i)
         {
             int N=100;
-            double dt = M_PI*2./N, r = delta[i];
+            double dt = pi*2./N, r = delta[i];
             for(int j=0;j<=N; ++j)
             {
                 double x = P[i].x+r*cos(dt*j);
@@ -1487,7 +1484,7 @@ int  WalkInTriangle(const Mesh & Th,int it, double *lambda,
 
 	if (k==2) // 2
 	{
-	    // let j be the vertex beetween the 2 edges
+	    // let j be the vertex between the 2 edges
 	    int j = 3-neg[0]-neg[1];
 	    R S = Area2(P,PF,Q[j]);
 
@@ -1813,10 +1810,16 @@ Mesh::Mesh(const Mesh & Th,int * split,bool WithMortar,int label)
 	nv =0;
 	quadtree = new FQuadTree(this,Pmin,Pmax,nv); // build empty the quadtree
         long iseuil =(long) (quadtree->coef*seuil);
-        if(verbosity>5)
-         cout << " seuil = " <<  seuil << " hmin = " << hm << " iseuil/ quadtree: " << iseuil <<  endl;
+        long iseuilhm = (long) (quadtree->coef*hm*0.8);
+        if( iseuilhm ==0)
+        {
+            cerr << " The generatated 2d mesh is to fine :  hmin = " << hm << " to to small < "<< 1./quadtree->coef  << endl;
+            ffassert(0); 
+        }
+        if( verbosity>5 && ! iseuil )
+         cout << " seuil = " <<  seuil << " hmin = " << hm << " iseuil/ quadtree: " << iseuil << " coef quadtree " <<  quadtree->coef << endl;
 
-       ffassert(iseuil); //  zero => too smal
+       //      ffassert(iseuil); //  zero => too smal
         {  // to keep the order of old vertices to have no problem in
              // interpolation on sub grid of RT finite element for example (Feb. 2016 F. Hecht)
         KN<bool> setofv(Th.nv,false);
@@ -1975,8 +1978,7 @@ Mesh::Mesh(const Mesh & Th,int * split,bool WithMortar,int label)
 		    R2 B=vertices[vt[1]];
 		    R2 C=vertices[vt[2]];
 		    R a = (( B-A)^(C-A))*0.5;
-                    if( a<0) nwarm++;
-                    if( nwarm<10 && verbosity>9) cout << " warning: bad oriantiation in trunc " << kt << " " << a << endl  ;
+                    if(a <0 &&  nwarm++ <10 && verbosity>9) cout << " warning: bad oriantiation in trunc " << kt << " " << a << endl  ;
 		    if (a>0)
 			triangles[kt].set(vertices,vt[0],vt[1],vt[2],T.lab);
 		    else
